# The plugin interface utilizes an extension point in the waif_show_file
# routine.  Currently, only ogg and mp3 hooks exist.  This can be changed 
# later without too much effort but I do not know of any other file types 
# that implement an id3/ogginfo-like header for artist/track identification.
#
# The calling procedure will feed a plugin procedure with two arguments:
#   the name of an array to store data in (this requires an upvar)
#   the full path of the filename
# 
# The basic template for defining your own procedure is:
# 
# proc show_mp3 { arrayname filename } {
#    upvar $arrayname myarray	;# this copies the array to local context
# 				;# changes made are made to the array in the
#				;# parent context, as well
#	
#    set externalcmd "id3ed -f \"$filename\""	;# the external cmd to parse
#						;# note the filename is quoted 
#
#    if [catch {open "|$externalcmd" r } fd] {	;# open a pipe to the external
#        return					;# cmd and return if pipe fails
#    }
#    while { [gets $fd line] > -1 } {		;# step through line by line
#	# your parsing code here
#	set myarray(artist) $artist	;# assuming you can parse the
#	set myarray(songname) $songname	;# artist and songname from 
#    }					;# the piped output
#	catch { close $fd }		;# close the pipe - no leaking fd's!!!
#	return $arrayname			
#    }				
# }
#
# The return value is not used at this point but I may check for a null return,
# e.g. just "return" as a sign of an error.  
# 
# The key piece of code above is populating the array with artist and songname
# elements.  If those elements are lacking from the array, only the filename
# will be displayed in the output (like vintage waif.)
# 
# The plugin procedure must be named show_(filetype) where filetype is either 
# ogg of mp3.  The easiest way to insure that your custom routine will be used
# over the defaults I provide here is to eliminate the default procedure. You
# can either delete it, or a much simpler method is to fake out the interpreter
# by enclosing the procedure definition in a call to set.  For example,
# 
# set dontusethis {
# proc helloworld { } {
# 	puts "hello world"
# }
# } ;# end set dontusethis
# 
# The interpreter will see the code above as variable assignment instead of 
# procedure definition.
# 
# FINALLY, THIS IS VERY IMPORTANT!
# To utilize a newly defined procedure, you must run the mkindex.sh script
# in this directory to update the procedure registry (tclIndex)

namespace eval ::Waif {
    namespace export show_mp3v1 show_mp3v2 show_ogg

# the default plugin routine for ogg utilizes ogginfo from vorbis-tools-rc1
proc show_ogg { arrayname filename } {
	upvar $arrayname myarray

	set externalcmd "ogginfo \"$filename\""
    if [catch { open "|$externalcmd" r } fd] {
        return
    }
    while { [gets $fd line] > -1 } {
        if [regexp "ARTIST=(.*)" $line all artist] {
            regsub "(\ *)$" $artist "" artist
			set myarray(artist) $artist
        }
        if [regexp "TITLE=(.*)" $line all songname] {
            regsub "(\ *)$" $songname "" songname
			set myarray(songname) $songname
        }
    }
    catch { close $fd }
	return $arrayname
}

# the default mp3 plugin routine utilizes id3ed available from freshmeat.net
proc show_mp3v1 { arrayname filename } {
	upvar $arrayname myarray

    set externalcmd "id3ed -i \"$filename\""
    if [catch { open "|$externalcmd" r } fd] {
        return
    }
    while { [gets $fd line] > -1 } {
        if { [regexp "(no tag)$" $line] } {
            return
        }
        if [regexp "songname: (.*)" $line all songname] {
            regsub "(\ *)$" $songname "" songname
			set myarray(songname) $songname
        }
        if [regexp "artist: (.*)" $line all artist] {
            regsub "(\ *)$" $artist "" artist
			set myarray(artist) $artist
        }
    }
    catch { close $fd }
	return $arrayname
}
 
# To enable ID3v2 support, add id3v2=1 to ~/.waifrc 
# You will need id3lib and id3v2 from freshmeat.net.
proc show_mp3v2 { arrayname filename } {
    upvar $arrayname myarray

    set externalcmd "id3v2 -l \"$filename\""
    if [catch { open "|$externalcmd" r } fd] {
        return
    }
    while { [gets $fd line] > -1 } {
        if { [regexp "(No ID3 tag)$" $line] } {
            catch { close $fd }
            return
        }
        if {[regexp "Title  : (.*) Artist: (.*)" $line - songname artist]} {
            regsub "( )*$" $artist "" artist
            regsub "( )*$" $songname "" songname
            set myarray(songname) $songname
            set myarray(artist)   $artist
        }
        if { [regexp "TPE1 (.*): (.*)" $line - - artist] } {
            set myarray(artist)   $artist
        }
        if {[regexp "TIT2 (.*): (.*)" $line - - songname]} {
            set myarray(songname) $songname
        }
    }
    catch { close $fd }
    return $arrayname
}

} ;# end of namespace 
