#!/bin/sh
# restart in expect - if we can't do this, there's no sense in proceeding \
exec expect "$0" "$@"

set curdir [pwd]
set email straycat000@yahoo.com
set install_version 0.59.7
set install_lib_dir [info library]
set install_man_dir /usr/local/man/man1
set install_bin_dir /usr/local/bin
set install_tar_file waiflib.tar
set install_man_file [file dirname [info script]]/waifsh.1
set install_tar [file dirname [info script]]/$install_tar_file
set install_bin_name waifsh
set install_lib_name Waif
set install_hlp_name waif-helper
set install_man_name waifsh.1
set install_tool_file [file dirname [info script]]/waifsh
set install_hlp_file [file dirname [info script]]/waif-helper

#------------------------------------------------------------------------
# Check who is running this script.	Insist on root.
#-----------------------------------------------------------------------
set who [eval exec whoami]
if { "$who" != "root" } {
	puts stderr "You must be root to run this script"
	exit 1
}
			
#-------------------------------------------------------------------------
# Make the yes or no answer a boolean.
#------------------------------------------------------------------------
proc yes { } {
	gets stdin rsp
	while {"$rsp" != "q"} {
		switch -regexp $rsp {
			^\[y|Y]$ { return 1 }
			^\[n|N]$ { return 0 }
			default {
				puts -nonewline stderr "Please answer \"y\" or \"n\" \[y|n]? "
				flush stdout
			}
		}
		gets stdin rsp
	}
}


#------------------------------------------------------------------------
# use my own which routine instead of making a syscall
#------------------------------------------------------------------------
proc which { app {debug 0} } {
	global env
	if $debug { puts "looking for $app :" }
	foreach dir [split $env(PATH) ":"] {
		if $debug { puts -nonewline "checking $dir... " ; flush stdout }
		if [file exists $dir/$app] {
			if $debug { puts "yes" }
			return $dir/$app
		} else {
			if $debug { puts "no" }
		}
	}
	return 
}

#------------------------------------------------------------------------
# Provide a no-brainer first question to give the timid or curious a chance
# to back out if they wish.
#------------------------------------------------------------------------
puts "This will install Waif v. $install_version."
puts -nonewline "Do you wish to continue? \[y|n]? " ; flush stdout
if ![yes] {
	exit 1
}

#-------------------------------------------------------------------------
# Check for an old waifsh to backup.
#-------------------------------------------------------------------------
set oldwaifsh [which waifsh]
if { "$oldwaifsh" != "" } {
	if { ("[file tail $oldwaifsh]" == "waifsh") && ("[file dirname $oldwaifsh]" != ".") } {
		set backup_previous 1
	} else {
		unset oldwaifsh
	}
} else {
	catch { unset oldwaifsh } 
}

#-------------------------------------------------------------------------
# Check for an existing installation by performing a package require.
# If successful, give a holler and see if Waif is to be reinstalled.
#-------------------------------------------------------------------------
if ![catch {package require Waif} previous_version] {
	puts "A previous installation was found (version $previous_version)."
	puts -nonewline "Do you wish to continue \[y|n]? " ; flush stdout
	if [yes] {
		set backup_previous 1
	} else {
		puts stderr "Aborting installation."
		exit 1
	}
} else {
	catch { unset previous_version } 
}
if ![info exists backup_previous] {
	set backup_previous 0
}

#--------------------------------------------------------------------------
# Confirm installation location.
#--------------------------------------------------------------------------
puts -nonewline "Install the library files in $install_lib_dir \[y|n]? " 
flush stdout
if ![yes] {
	puts "*WARNING* deviating from the detected Tcl library directory may"
	puts "affect your ability to use waif.	You have been warned!"
	puts -nonewline "Path to install libraries: " ; flush stdout
	gets stdin install_lib_dir
}
puts -nonewline "Install waifsh in $install_bin_dir \[y|n]? "
flush stdout
if ![yes] {
	puts -nonewline "Path to install waifsh: " ; flush stdout
	gets stdin install_bin_dir
}
puts "Confirm the installation locations"
puts ""
puts "Library install directory:	 $install_lib_dir"
puts "Waif install directory:		 $install_bin_dir"
puts ""
puts -nonewline "Is this correct? \[y|n]? " ; flush stdout
if ![yes] {
	puts stderr "Please re-run this install script.	Aborting."
	exit 1
}

#--------------------------------------------------------------------------
# Backup previous if necessary
#--------------------------------------------------------------------------
if {$backup_previous} {
	puts "Backing up previous installation..."
	if [info exists oldwaifsh] {
		puts "Renaming old waifsh ($oldwaifsh) to $oldwaifsh.bak"
		file rename -force $oldwaifsh $oldwaifsh.bak
	}
	if [file exists [info library]/Waif/Waif.tcl] {
		cd [info library]
		# check for an existing backup - rename if necessary
		eval exec tar cvf [file dirname [info script]]/$install_tar_file.old \
			Waif 
		puts "Lib directory backed up to [info library]/$install_tar_file.old"
#---------------------------------------------------------------------------
# danger! danger! deleting the waif library directory
#---------------------------------------------------------------------------
		# debug
		puts "Deleting previous installation"
		eval exec rm -rf [info library]/Waif
	}
}

#---------------------------------------------------------------------------
# Check the paths and install
#---------------------------------------------------------------------------
foreach dir [list install_lib_dir install_bin_dir] {
	if ![file isdirectory $install_lib_dir] {
		puts "Cannot find $dir.	Create the directory and run this script again"
		exit 1
	}
}

# change back to the source directory
cd $curdir

# first the libraries
if [file exists $install_lib_dir/$install_tar_file] {
	file rename -force $install_lib_dir/$install_tar_file $install_lib_dir/$install_tar_file.old
}
puts "Installing library  : $install_lib_name"
file copy $install_tar $install_lib_dir
cd $install_lib_dir
eval exec tar xvf $install_tar_file

# next the tools
cd $curdir
puts "Installing program  : $install_bin_name"
file copy -force $install_tool_file $install_bin_dir
puts "Installing helper   : $install_hlp_name"
file copy -force $install_hlp_file $install_bin_dir
puts "Installing man page : $install_man_name"
file copy -force $install_man_file $install_man_dir

#------------------------------------------------------------------------
# Perform post-install checks.
# Create a postinstall interpreter and load Waif in it.  If all goes 
# well, we're in business.
#------------------------------------------------------------------------

puts "Performing post-install checks."
set unsuccessful 0
interp create postinstall
if { ([catch { interp eval postinstall package require Waif } version]) } {
	puts "Installation was not successful.  Could not load Waif package."
	set unsuccessful 1
}
if { "$version" != "$install_version" } {
	puts "Waif installation was not successful." 
	puts "Package loaded does not match package installed ($version vs. $install_version)."
	set unsuccessful 1
}
if $unsuccessful {
	puts "Please re-run the install script or send a bug report to $email"
	exit 1
} else {
	if [info exists previous_version] {
		set install_major [lindex [split $previous_version "."] 1]
		if { $install_major < 59 } {
			puts "The MIME file has changed in 0.59."
			puts "Please remove ~/.waif/mime.  A new one will be created."
		}
	}
puts "
****************************************************************
 Many bugs have been added, all your favorite features	
 have been removed, and most importantly, you should read 
 the WHATSNEW file before you begin.  In short, things have 
 have changed.  Save yourself some grief. :)
****************************************************************
"
	puts "Installation Successful."
}
exit

